package com.aitrainer.api.controller

import com.aitrainer.api.model.CustomerProperty
import com.aitrainer.api.repository.CustomerPropertyRepository
import org.slf4j.LoggerFactory
import org.springframework.http.ResponseEntity
import org.springframework.security.access.annotation.Secured
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class CustomerPropertyController(private val customerPropertyRepository: CustomerPropertyRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @PostMapping("/customer_property")
    fun createNewCustomerProperty(@Valid @RequestBody customerProperty: CustomerProperty): ResponseEntity<CustomerProperty> {
        logger.info("Create customer property: $customerProperty")
        return ResponseEntity.ok().body(customerPropertyRepository.save(customerProperty))
    }

    @Secured
    @PostMapping("customer_property/update/{id}")
    fun updateCustomerProperty(@PathVariable(value = "id") propertyId: Long,
                               @Valid @RequestBody propertyToUpdate: CustomerProperty): ResponseEntity<CustomerProperty> {
        val customerProperty = customerPropertyRepository.findById(propertyId).orElse(null)
            ?: return ResponseEntity.notFound().build()

        val updatedCustomerProperty = customerProperty.copy(
            propertyValue = propertyToUpdate.propertyValue,
            dateAdd = propertyToUpdate.dateAdd
        )
        return ResponseEntity.ok().body(customerPropertyRepository.save(updatedCustomerProperty))
    }

    @GetMapping("/customer_property/{id}")
    fun getAllByCustomerId(@PathVariable(value = "id") customerId: Long): ResponseEntity<List<CustomerProperty>> {

        val propertyList: List<CustomerProperty>  = customerPropertyRepository.findAllByCustomerId(customerId)
        logger.info("Get all customer_property by customerId")

        return if(propertyList.isNotEmpty())
            ResponseEntity.ok().body(propertyList) else
            ResponseEntity.notFound().build()
    }

    @GetMapping( "/customer_property/last/{id}")
    fun getLastPropertiesByCustomerId(@PathVariable(value = "id") customerId: Long): ResponseEntity<List<CustomerProperty>> {
        val propertyList = customerPropertyRepository.findLastPropertiesByCustomerId(customerId)
        logger.info("Get last customer_property by customerId $propertyList")

        return if (propertyList.isNotEmpty())
            ResponseEntity.ok().body(propertyList) else
            ResponseEntity.notFound().build()
    }
}