package com.aitrainer.api.controller

import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import org.slf4j.LoggerFactory
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.GetMapping
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController
import com.google.gson.GsonBuilder


@RestController
@RequestMapping("/api")
class PackageController(private val exerciseAbilityRepository: ExerciseAbilityRepository,
                        private val exerciseTypeRepository: ExerciseTypeRepository,
                        private val exerciseTreeRepository: ExerciseTreeRepository,
                        private val propertyRepository: PropertyRepository,
                        private val productRepository: ProductRepository,
                        private val exerciseDeviceRepository: ExerciseDeviceRepository,
                        private val exerciseTreeParentsRepository: ExerciseTreeParentsRepository,
                        private val exercisePlanTemplateRepository: ExercisePlanTemplateRepository,
                        private val evaluationRepository: EvaluationRepository,
                        private val sportRepository: SportRepository,
                        private val tutorialRepository: TutorialRepository
) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @GetMapping("/app_package")
    fun getPackageData(): ResponseEntity<String> {

        val gson = GsonBuilder()
            .excludeFieldsWithoutExposeAnnotation()
            .setPrettyPrinting()
            .create()

        val listDevices: List<ExerciseDevice> = exerciseDeviceRepository.getExerciseDevices()
        val listDevicesJson: String = gson.toJson(listDevices)

        val listProducts = productRepository.findAll()
        val listProductsJson: String = gson.toJson(listProducts)

        val listProperty:List<Property> = propertyRepository.getProperties()
        val listPropertyJson: String = gson.toJson(listProperty)

        val listExerciseTree = exerciseTreeRepository.getActiveMenu()
        val listExerciseTreeJson: String = gson.toJson(listExerciseTree)

        val listExerciseType = exerciseTypeRepository.findAll()
        val listExerciseTypeJson = gson.toJson(listExerciseType)

        val listExerciseAbility = exerciseAbilityRepository.findAll()
        val listExerciseAbilityJson: String = gson.toJson(listExerciseAbility)

        val listExerciseTreeParents = exerciseTreeParentsRepository.findAll()
        val listExerciseTreeParentsJson: String = gson.toJson(listExerciseTreeParents)

        val listPlanTemplate = exercisePlanTemplateRepository.findAll()
        val listPlanTemplateJson: String = gson.toJson(listPlanTemplate)

        val listEvaluations = evaluationRepository.findAll()
        val listEvaluationJson: String = gson.toJson(listEvaluations)

        val listSports = sportRepository.getSports()
        val listSportsJson: String = gson.toJson(listSports)

        val listTutorials = tutorialRepository.findAll()
        val listTutorialJson: String = gson.toJson(listTutorials)

        val packageJson: String =
                    getClassRecord(ExerciseDevice::class.simpleName, listDevicesJson) +
           "|||" +  getClassRecord(Product::class.simpleName, listProductsJson) +
           "|||" +  getClassRecord(Property::class.simpleName, listPropertyJson) +
           "|||" +  getClassRecord(ExerciseTree::class.simpleName, listExerciseTreeJson) +
           "|||" +  getClassRecord(ExerciseType::class.simpleName, listExerciseTypeJson) +
           "|||" +  getClassRecord(ExerciseAbility::class.simpleName, listExerciseAbilityJson) +
           "|||" +  getClassRecord(ExerciseTreeParents::class.simpleName, listExerciseTreeParentsJson) +
           "|||" +  getClassRecord(ExercisePlanTemplate::class.simpleName, listPlanTemplateJson) +
           "|||" +  getClassRecord(Evaluation::class.simpleName, listEvaluationJson) +
           "|||" +  getClassRecord(Sport::class.simpleName, listSportsJson) +
           "|||" +  getClassRecord(Tutorial::class.simpleName, listTutorialJson)

        return if (packageJson.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(packageJson)
    }

    fun getClassRecord(className: String?, json: String): String {
        return "$className***$json"
    }
}