package com.aitrainer.api.controller

import com.aitrainer.api.model.ProductTest
import com.aitrainer.api.repository.ProductTestRepository
import org.slf4j.LoggerFactory
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class ProductTestController(private val productTestRepository: ProductTestRepository) {
    private val logger = LoggerFactory.getLogger(javaClass)

    @GetMapping("/product_test/customer/{id}")
    fun getAllByCustomerId(@PathVariable(value = "id") customerId: Long): ResponseEntity<List<ProductTest>> {

        val productTestList  = productTestRepository.findByCustomerId(customerId)
        logger.info("Get all productTest by customerId $productTestList")

        return if(productTestList.isNotEmpty())
            ResponseEntity.ok().body(productTestList) else
            ResponseEntity.notFound().build()
    }

    @PostMapping("/product_test")
    fun insertProductTest(@Valid @RequestBody productTest: ProductTest): ResponseEntity<ProductTest> {
        val newProductTest = productTestRepository.save(productTest)
        logger.info("Create new productTest: $newProductTest")
        return ResponseEntity.ok().body(newProductTest)
    }
}