package com.aitrainer.api.test

import com.aitrainer.api.controller.PackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.ResponseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppPackageTest {

    @Autowired
    private lateinit var exerciseAbilityRepository: ExerciseAbilityRepository
    @Autowired
    private lateinit var exerciseTypeRepository: ExerciseTypeRepository
    @Autowired
    private lateinit var  exerciseTreeRepository: ExerciseTreeRepository
    @Autowired
    private lateinit var  propertyRepository: PropertyRepository
    @Autowired
    private lateinit var  productRepository: ProductRepository
    @Autowired
    private lateinit var  exerciseDeviceRepository: ExerciseDeviceRepository
    @Autowired
    private lateinit var exerciseTreeParentsRepository: ExerciseTreeParentsRepository
    @Autowired
    private lateinit var exercisePlanTemplateRepository: ExercisePlanTemplateRepository
    @Autowired
    private lateinit var evaluationRepository: EvaluationRepository
    @Autowired
    private lateinit var sportRepository: SportRepository
    @Autowired
    private lateinit var tutorialRepository: TutorialRepository

    @Test
    fun testAppPackage() {
        val gson = Gson()

        val controller = PackageController(
            exerciseAbilityRepository,
            exerciseTypeRepository,
            exerciseTreeRepository,
            propertyRepository,
            productRepository,
            exerciseDeviceRepository,
            exerciseTreeParentsRepository,
            exercisePlanTemplateRepository,
            evaluationRepository,
            sportRepository,
            tutorialRepository
        )
        val response: ResponseEntity<*> = controller.getPackageData()

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            if ( record[0] == ExerciseType::class.simpleName) {
                //print("List ExerciseType: " + record[1])
                val exerciseTypeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseType?>?>() {}.type
                val listExerciseType: List<ExerciseType> = gson.fromJson(exerciseTypeJson, type)
                assertTrue(listExerciseType.isNotEmpty())
                assertEquals(listExerciseType[1].name, "Húzódzkodás")
                //assertEquals(listExerciseType[1].translations[0].name, "Bicepsz")
            } else if (record[0] == ExerciseTree::class.simpleName) {
                val exerciseTreeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTree?>?>() {}.type
                val listExerciseTree: List<ExerciseTree> = gson.fromJson(exerciseTreeJson, type)
                assertTrue(listExerciseTree.isNotEmpty())
                assertEquals(listExerciseTree[1].name, "Strength")
                assertEquals(listExerciseTree[1].translations[0].name, "Erő!")
            } else if (record[0] == Property::class.simpleName) {
                val propertyJson: String = record[1]
                val type = object : TypeToken<List<Property?>?>() {}.type
                val listProperty: List<Property> = gson.fromJson(propertyJson, type)
                assertTrue(listProperty.isNotEmpty())
                assertEquals(listProperty[2].propertyName, "Chest")
                assertEquals(listProperty[0].translations[0].propertyName, "Tömeg")
            } else if (record[0] == ExerciseTreeParents::class.simpleName) {
                val exerciseTreeParentsJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTreeParents?>?>() {}.type
                val listParents: List<ExerciseTreeParents> = gson.fromJson(exerciseTreeParentsJson, type)
                assertTrue(listParents.isNotEmpty())
                assertEquals(listParents[2].exerciseTreeParentId, 0)
                assertEquals(listParents[6].exerciseTreeParentId, 4)
                assertEquals(listParents[6].exerciseTreeChildId, 9)
            } else if (record[0] == ExercisePlanTemplate::class.simpleName) {
                val exercisePlanTemplateJson: String = record[1]
                val type = object : TypeToken<List<ExercisePlanTemplate?>?>() {}.type
                val listTemplate: List<ExercisePlanTemplate> = gson.fromJson(exercisePlanTemplateJson, type)
                assertTrue(listTemplate.isNotEmpty())
                assertEquals(listTemplate[0].name, "Own Body")
                assertEquals(listTemplate[0].templateType, "mini_test_set")
                assertEquals(listTemplate[0].translations[0].name, "Saját testes tesztek")
                assertEquals(listTemplate[0].details[0].exerciseTypeId, 2)
                assertEquals(listTemplate[1].details[1].exerciseTypeId, 33)
                assertEquals(listTemplate[1].details[1].sort, 2)
            } else if (record[0] == Evaluation::class.simpleName) {
                val evaluationJson: String = record[1]
                val type = object : TypeToken<List<Evaluation?>?>() {}.type
                val evaluations: List<Evaluation> = gson.fromJson(evaluationJson, type)
                assertTrue(evaluations.isNotEmpty())
                assertEquals(evaluations.size, 2)
                assertEquals(evaluations[0].name, "PushUps")
                assertEquals(evaluations[0].attributes.size, 18)
                assertEquals(evaluations[0].attributes[1].name, "Fekvőtámasz_ffi_17-19_fair")
            } else if (record[0] == Sport::class.simpleName) {
                val sportJson: String = record[1]
                val type = object : TypeToken<List<Sport?>?>() {}.type
                val sports: List<Sport> = gson.fromJson(sportJson, type)
                assertEquals(sports.size, 4)
                assertEquals(sports[1].name, "Football")
                assertEquals(sports[1].translations[0].sportName, "Labdarúgás")
                assertEquals(sports[2].name, "Footgolf")
                assertEquals(sports[2].translations[0].sportName, "Footgolf")
                assertEquals(sports[3].translations[0].sportName, "Tenisz")
            }  else if (record[0] == Tutorial::class.simpleName) {
                print("List Tutorial: " + record[1])
                val tutorialJson: String = record[1]
                val type = object : TypeToken<List<Tutorial?>?>() {}.type
                val tutorials: List<Tutorial> = gson.fromJson(tutorialJson, type)
                assertEquals(tutorials.size, 1)
                assertEquals(tutorials[0].name, "Basic")
                assertEquals(tutorials[0].steps.size, 5)
                assertEquals(tutorials[0].steps[3].parent_id, 2)
                assertEquals(tutorials[0].steps[3].condition, "Check2")
                assertEquals(tutorials[0].steps[2].direction, "up|88")
                assertEquals(tutorials[0].steps[3].action, 0)
                assertEquals(tutorials[0].steps[4].action, 1)
                assertEquals(tutorials[0].steps[4].branch, "Check1")
                assertEquals(tutorials[0].steps[0].translations.size, 2)
            }
        }



    }
}