package com.aitrainer.api.test


import com.aitrainer.api.controller.ProductTestController
import com.aitrainer.api.repository.ProductTestRepository
import org.junit.jupiter.api.Test
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import kotlin.test.assertEquals
import kotlin.test.assertTrue
import com.aitrainer.api.model.ProductTest
import org.junit.jupiter.api.TestInstance


@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class ProductTestTest {

    @Autowired
    private lateinit var productTestRepository: ProductTestRepository


    @Test
    fun testGetProductTests() {
        val list = productTestRepository.findByCustomerId(1)
        print("List $list")

        val controller = ProductTestController(productTestRepository)
        val response = controller.getAllByCustomerId(62)

        val productTest = response.body

        print ("resp $response.body" )

        assertTrue(productTest is List)
        assertTrue(productTest.isNotEmpty())
        assertEquals(productTest.size, 2)
        assertEquals(productTest[0].productId, 2)
        assertEquals(productTest[0].dateView, "2020-10-01 12:00:00")

    }

    @Test
    fun testInsertProductTest() {
        val productTest = ProductTest(
            customerId = 2,
            productId = 2,
            dateView = "2020-11-01 12:00:00",
            purchaseClick = false
        )
        val controller = ProductTestController(productTestRepository)
        val response = controller.insertProductTest(productTest)

        val productTestNew = response.body

        assertTrue(productTestNew is ProductTest)
        assertEquals(productTestNew.productId, 2)
        assertEquals(productTestNew.customerId, 2)

        productTestRepository.delete(productTestNew)

    }

}